Class("CheckBox") ({

	["load"] = function(s)
		table.insert(GUIs,s)
		s.size = {32,32}
	end,

	["checked"] = false,
	
	["changeState"] = function(s,Args) 
		s.checked = Args or not s.checked
		s.checkStateChanged:runEvents(s.checked)
	end,
	
	["draw"] = function(s,Args)
		local size = s.size
		local position = s.position
		local scaleX, scaleY = size[1]/32, size[2]/32
		
		local pr,pg,pb,al = love.graphics.getColor()
		love.graphics.setColor(s.backgroundColor[1],s.backgroundColor[2],s.backgroundColor[3])
		love.graphics.draw(GUIGraphics["Backing"],position[1],position[2],0,size[1],size[2])	
		love.graphics.setColor(pr,pg,pb,al)
		if s.checked then	
			love.graphics.draw(GUIGraphics["Check"],position[1],position[2],0,scaleX,scaleY)	
		end
	end,
	
	["update"] = function(s)
		if s.mouseJustClicked then
			s:changeState()
		end
		
		if s.backgroundColor[1] < 60 and s.backgroundColor[2] < 60 and s.backgroundColor[3] < 60 then
			s.backgroundColor = {60,60,60}
		end
	end,
	
	["checkStateChanged"] = {
		["connect"] = function(s,connectFunc)
			table.insert(s.callBacks,connectFunc)
		end,
		["runEvents"] = function(s,checked)
			for _,v in pairs(s.callBacks) do
				coroutine.resume(coroutine.create(function()
					v(checked)
				end))
			end
		end,
		callBacks = {},
	},
	
	["mousePressed"] = {
		["connect"] = function(s,connectFunc)
			table.insert(s.callBacks,connectFunc)
		end,
		["runEvents"] = function(s,x,y)
			for _,v in pairs(s.callBacks) do
				coroutine.resume(coroutine.create(function()
					v(x,y)
				end))
			end
		end,
		callBacks = {},
	},
	
	["mouseReleased"] = {
		["connect"] = function(s,connectFunc)
			table.insert(s.callBacks,connectFunc)
		end,
		["runEvents"] = function(s,x,y)
			for _,v in pairs(s.callBacks) do
				coroutine.resume(coroutine.create(function()
					v(x,y)
				end))
			end
		end,
		callBacks = {},
	},
	
	["mouseClicked"] = {
		["connect"] = function(s,connectFunc)
			table.insert(s.callBacks,connectFunc)
		end,
		["runEvents"] = function(s,x,y)
			for _,v in pairs(s.callBacks) do
				coroutine.resume(coroutine.create(function()
					v(x,y)
				end))
			end
		end,
		callBacks = {},
	},
	
	["mouseEntered"] = {
		["connect"] = function(s,connectFunc)
			table.insert(s.callBacks,connectFunc)
		end,
		["runEvents"] = function(s,x,y)
			for _,v in pairs(s.callBacks) do
				coroutine.resume(coroutine.create(function()
					v(x,y)
				end))
			end
		end,
		callBacks = {},
	},
	
	["mouseExited"] = {
		["connect"] = function(s,connectFunc)
			table.insert(s.callBacks,connectFunc)
		end,
		["runEvents"] = function(s,x,y)
			for _,v in pairs(s.callBacks) do
				coroutine.resume(coroutine.create(function()
					v(x,y)
				end))
			end
		end,
		callBacks = {},
	}
	
	}, {GUIElement})