Class("Changeable") {update, changeState}
Class("Drawable") {draw}

Class("GUIElement") (
					{
						["size"] = {0,0}, 
						["position"] = {0,0},
						["hasCaption"] = false,
						["caption"] = "GUIElement",
						["captionTextColor"] = {255,0,0},
						["captionBackgroundColor"] = {0,255,255},
						["showingCaption"] = false,
						["maxCaptionCountdown"] = 40,
						["captionCountdown"] = 40,
						["hasBorder"] = true,
						["borderColor"] = {0,0,0},
						["backgroundColor"] = {0,0,0},
						["borderWidth"] = 3,
						["mouseHovering"] = false,
						["mouseDown"] = false,
						["mouseJustClicked"] = false,
						["mouseJustReleased"] = false,
						["hasShading"] = true,
						["maxMouseClickedReleaseTime"] = 15,
						["mouseClickedReleaseTime"] = 15,	--Amount of frames to not trigger mouseClick event
						["drawBorder"] = function(s)
							if s.hasBorder then
								drawBorder(s.position[1], s.position[2], s.position[1]+s.size[1], s.position[2]+s.size[2], s.borderWidth, s.borderColor)
							end
						end,
						
						["drawShading"] = function(s)
							if s.hasShading and s.mouseHovering and not s.mouseDown then
								love.graphics.draw(GUIGraphics.Shading, s.position[1], s.position[2], 0, s.size[1], s.size[2])
							elseif s.mouseDown and s.hasShading then
								love.graphics.draw(GUIGraphics.IntenseShading, s.position[1], s.position[2], 0, s.size[1], s.size[2])
							end
						end,
						
						["drawCaption"] = function(s)
							local font = love.graphics.getFont()
							if s.hasCaption and s.showingCaption then
								local pr,pg,pb,al = love.graphics.getColor()
								local width = font:getWidth(s.caption)
								local height = font:getHeight(s.caption)
								local paddingX = 5
								local paddingY = 5
								local x = love.mouse.getX() + 10
								local y = love.mouse.getY() + 10
								love.graphics.setColor(s.captionBackgroundColor[1], s.captionBackgroundColor[2], s.captionBackgroundColor[3])
								love.graphics.rectangle("fill", x, y, width+paddingX, height+paddingY)
								love.graphics.setColor(s.captionTextColor[1], s.captionTextColor[2], s.captionTextColor[3])
								love.graphics.print(s.caption, x+paddingX/2, y+paddingY/2)
								love.graphics.setColor(pr,pg,pb,al)
								drawBorder(x, y, x+width+paddingX, y+height+paddingY, 1, {0,0,0})
							end
						end,
						
						["gUpdate"] = function(s)
							--Runs update code that almost all GUI objects need.
							local mC = s.mouseDown
							local mX, mY = love.mouse.getPosition()
							--It may seem inefficient to have two if statements, but it actually halves the math in some cases!
							if mX > s.position[1] and mX < s.position[1] + s.size[1] then
								if mY > s.position[2] and mY < s.position[2] + s.size[2] then	
									if not mouseDownCache and love.mouse.isDown("l") then
										s.mouseJustClicked = true
										s.mouseDown = true
										s.captionCountdown = s.captionCountdown - 1
									elseif love.mouse.isDown("l") then
										s.mouseJustClicked = false
										s.mouseDown = true
										s.captionCountdown = s.captionCountdown - 1
									else
										s.mouseJustClicked = false
										s.mouseDown = false
										s.captionCountdown = s.captionCountdown - 1
									end
									if s.mouseHovering == false and s.mouseEntered ~= nil then
										s.mouseEntered:runEvents(mX,mY)
									end
									s.mouseHovering = true
									s.captionCountdown = s.captionCountdown - 1
								else
									s.mouseJustClicked = false
									if s.mouseHovering == true and s.mouseExited ~= nil then
										s.mouseExited:runEvents(mX,mY)
									end
									s.mouseHovering = false
									s.mouseDown = false
									s.captionCountdown = s.maxCaptionCountdown
								end
							else
								s.mouseJustClicked = false
								if s.mouseHovering == true and s.mouseExited ~= nil then
									s.mouseExited:runEvents(mX,mY)
								end
								s.mouseHovering = false
								s.mouseDown = false
								s.captionCountdown = s.maxCaptionCountdown
							end
							
							if s.captionCountdown <= 0 then
								s.showingCaption = true
							else
								s.showingCaption = false
							end
							
							if mC and not s.mouseDown then
								s.mouseJustReleased = true
							else
								s.mouseJustReleased = false
							end
							
							--Mouse events
						
							if s.mouseJustClicked then
								s.mouseClickedReleaseTime = s.maxMouseClickedReleaseTime
								if s.mousePressed ~= nil then
									s.mousePressed:runEvents(mX,mY)
								end
							elseif s.mouseJustReleased then
								if s.mouseReleased ~= nil then
									s.mouseReleased:runEvents(mX,mY)
								end
								if s.mouseClickedReleaseTime > 0 then
									if s.mouseClicked ~= nil then
										s.mouseClicked:runEvents(mX,mY)	--Register as a click if they pressed and released fast enough
									end
								end
							end
							
							if s.mouseDown then
								s.mouseClickedReleaseTime = s.mouseClickedReleaseTime - 1
							end
						end,
						
					}, {Changeable, Drawable})
